#!/bin/bash
# Pyra Testing Script v0.6

SCRIPTPATH="$(dirname "$0")"
LOG_TTY=3
LOG_DEVICE="/dev/tty$LOG_TTY"


ver="2018-11-12"

sd_dir=`pwd`

# Setup some functions

log()
{
	echo "$@"
	echo "$@" > $LOG_DEVICE
}

log_err()
{ 
	echo -en "\033[1;31m"
	echo "$@"
	echo -en "\033[0m"

	echo -en "\033[1;31m" > $LOG_DEVICE
	echo "$@" > $LOG_DEVICE
	echo -en "\033[0m" > $LOG_DEVICE
}

print_ok()
  {
    if $2; then
      log "$1 OK"
    else
      log_err "$1 FAILED"
    fi
  }
  print_status()
  {
    log      "---------------------"
    print_ok "Barometer:       " $barometer_passed
    print_ok "Compass:         " $compass_passed
    print_ok "Gyroscope:       " $gyro_passed
    print_ok "Fuel Gauge:      " $fuelgauge_passed
    print_ok "TWL4030:         " $twl_passed
    print_ok "LED Controllers: " $ledc_passed
    print_ok "Nub Controllers: " $nubs_passed
    print_ok "4G/LTE:          " $wwan_passed
    print_ok "WiFi:            " $wlan_passed
         log "Pyra Version:    " $pyr_version
    log      "---------------------"
  }

  
# Setup some directories we might not have yet

[ -d /dev ] || mkdir -m 0755 /dev
[ -d /root ] || mkdir -m 0700 /root
[ -d /sys ] || mkdir /sys
[ -d /proc ] || mkdir /proc
[ -d /tmp ] || mkdir /tmp

mkdir -p /var/lock
mount -t sysfs -o nodev,noexec,nosuid sysfs /sys
mount -t proc -o nodev,noexec,nosuid proc /proc
mount -t devtmpfs -o nosuid,mode=0755 udev /dev
mkdir /dev/pts
mount -t devpts -o noexec,nosuid,gid=5,mode=0620 devpts /dev/pts || true
mount -t tmpfs -o "noexec,nosuid,size=10%,mode=0755" tmpfs /run

mount -o remount,rw / /

# Load the needed modules
modprobe omapdss
modprobe omapdrm
modprobe encoder_tpd12s015
modprobe connector_hdmi
modprobe panel_boe_w677l
modprobe pwm_bl
modprobe leds_is31fl319x
modprobe bno055
modprobe pwm_omap_dmtimer
modprobe tsc2007
modprobe tca8418_keypad
modprobe palmas_pwrbutton
modprobe as5013
modprobe ehci_omap
modprobe bmp280_i2c
modprobe bq27xxx_battery_i2c
modprobe wlcore_sdio
modprobe wl18xx    

rfkill unblock wwan

# Disable most output on the LCD and switch to TTY3, use a nicer font size, siwth off all LEDs.
echo 0 > /sys/module/printk/parameters/ignore_loglevel
clear
setfont Lat15-Terminus24x12.psf.gz
chvt 3
dmesg -D
clear

  cd /sys/class/leds
        for i in *
        do
                echo $i
                echo "none" >$i/trigger
                echo 0 >$i/brightness
        done
cd $SCRIPTPATH
echo 0 > /sys/class/backlight/keyboard-backlight/brightness 

log ""
log "Pyra Testing Script $ver"
log "" > $LOG_DEVICE
log "This is a work-in-progress - we're getting there :))"
log ""


# Setting all to false :)

barometer_passed=false
compass_passed=false
gyro_passed=false
fuelgauge_passed=false
ledc_passed=false
nubs_passed=false
twl_passed=false
wwan_passed=false
wlan_passed=false

# Tests will run here.
cd $SCRIPTPATH
log "Testing Barometer..."
  if ./letux/hw-test -q -r barometer;    then barometer_passed=true; fi
  log "Testing Compass..."
  if ./letux/hw-test -q -r bno055_compass;    then compass_passed=true; fi
  log "Testing Gyro..."
  if ./letux/hw-test -q -r bno055_gyro;    then gyro_passed=true; fi
  log "Testing Nub-Controller..."
  if ./letux/hw-test -q -r nubs;    then nubs_passed=true; fi
  log "Testing Fuel Gauge..."
  if ./letux/hw-test -q -r fuel_gauge;    then fuelgauge_passed=true; fi
  log "Testing TWL4030..."
  if ./letux/hw-test -q -r power_twl4030;    then twl_passed=true; fi
  log "Testing LED Controllers..."
  if ./letux/hw-test -q -r leds;    then ledc_passed=true; fi
  log "Starting WLAN-Test..."
    essid="PND_TEST"
    testfile="ftp://192.168.1.17/AiDisk_a1/share/test"
  log "Waiting for module..."
        for a in `seq 20`
            do
            if ifconfig -a | grep -q wlan0
            then
		break
            fi
            sleep 4
        done

        sleep 3

        if ! ifconfig -a | grep -q wlan0
            then
            log "Failed!"
            log "Something is wrong with the WLAN-Module!!"
        else
        ifconfig wlan0 up
        log "Connecting to the test server ($essid)..."
        sleep 3
        iwlist wlan0 scan
        sleep 3
        iwconfig wlan0 mode managed
        iwconfig wlan0 essid $essid

        for a in `seq 20`
            do
	if ! iwconfig wlan0 | grep -q 'Not-Associated'
	then
		break
	fi
        sleep 1
        done

        if iwconfig wlan0 | grep -q 'Not-Associated'
            then
	log "Couldn't connect to  $essid..."
	log "Is the router running?"
        else
        
            log "Waiting for IP Address (DHCP)..."

            mkdir -p /var/lib/dhcp
            if ! dhclient -1 wlan0
            then
                log "Didn't receive an IP-Address..."
            else
                log "Downloading testfile..."

            if ! wget $testfile -O /dev/null
            then
                log "Testfile couldn't be downloaded!"
                log "Is the USB-Stick in the router?"
            else
                wlan_passed=true
            fi
        fi
    fi
  fi
  killall dhclient 
  log "Quicktest 4G/LTE Modem..."
  if ./letux/hw-test test_wwan_usb >> /root/log.txt; then wwan_passed=true; fi
  log "Switching all LEDs to white..."
  cd /sys/class/leds
        for i in *
        do
                echo $i
                echo "default-on" >$i/trigger
                echo 255 >$i/brightness
        done
cd $SCRIPTPATH
echo 7 > /sys/class/backlight/keyboard-backlight/brightness 

log ""
log "Check that they are all white now!"
log "Enter to continue"
read a < $LOG_DEVICE

log ""
log "Switching off all LEDs..."
cd /sys/class/leds
        for i in *
        do
                echo $i
                echo "none" >$i/trigger
                echo 0 >$i/brightness
        done
  cd $SCRIPTPATH

log ""
log "Check that they are all switched off now (except for the Power one if connected to AC)"
log ""
  
pyr_version=$(tr -d '\0' </proc/device-tree/model 2>/dev/null)

# Show the Results...

log ""

print_status
if ! $barometer_passed || ! $compass_passed || ! $gyro_passed || ! $fuelgauge_passed || \
     ! $ledc_passed || ! $nubs_passed || ! $twl_passed || ! $wlan_passed
  then
    log ""
    log_err "======================================================================================="
    log_err "Warning! Some critical tests have failed! Please press ENTER to start the input-tester."
    log_err "======================================================================================="
  fi

log "Please press ENTER to start the input-tester."
read a < $LOG_DEVICE

/usr/share/pyra/bin/pyra_test_inputs


# Switching the internal port to the eMMC
./letux/txs e

# Let's flash to the eMMC :)

log ""
log "Now flashing the OS onto the eMMC..."
read -a MEMINFO < /proc/meminfo
RAM=$(expr ${MEMINFO[1]} / 1000000)
log "Pyra model : ${pyr_version}"
log "Ram: $RAM GB"
log " "
log "Checking for OS image..."
mkdir -p /media/sd2
mount /dev/mmcblk5p1 /media/sd2

ROOTFS=""
if [ -f "/media/sd2/rootfs.tgz" ] ; then
	ROOTFS=/media/sd2/rootfs.tgz
elif [ -f "/rootfs.tgz" ] ; then
	ROOTFS=/rootfs.tgz
fi

UBOOT=/usr/share/pyra/u-boot/pyra-u-boot-${RAM}g/

if [ -n $ROOTFS ] ; then
	log "Found $ROOTFS , now starting flashing..."
	chvt 1
	INITHWTEST=1 ./pyra-install.sh $ROOTFS $UBOOT /dev/mmcblk0
	chvt 3
	log ""
	log "Done. Finished flashing!"
	log "Don't forget to remove the SD Cards after shutdown."
	log ""
else
	log_err "The OS could not be found!"
	log "Please ensure the card is inserted and has the OS image file installed."
	log ""
fi

umount /media/sd2
rm -d /media/sd2

log "Press ENTER to shutdown."
# Or s+enter to go into a shell
log " "
read a < $LOG_DEVICE
if [[ a=="s" ]] ; then
	chvt 1
	/bin/bash
fi
poweroff -f

